#!/usr/bin/env python

# set of functions for fool-proof handling of google results pages.
# Do not use this file directly. Use compare_pages.py and print_parse.py instead!


import sys
import os
import socket
import cgi # necessary for Python < 2.6
import urllib # for quote() and quote_plus()
import urllib2
import urlparse
import httplib
import cookielib
import StringIO, gzip, zlib

def find_all(html, begin, end, string, name, positions):
    x = html.find(string, begin)
    while x != -1 and x < end:
        positions.append((x, name))
        x = html.find(string, x + len(string))

def is_good_url(url):
    netloc = url[1]
    path = url[2]
    if netloc:
        if 'www.google' in netloc and ('/search' in path or '/aclk' in path): return False
        if 'webcache.googleusercontent' in netloc or 'adwords.google' in netloc: return False
        return True
    if path and path.startswith('/interstitial'): return True
    return False

def parse_google_redirect(link, url, parameter):
    qs = cgi.parse_qs(url[4])
    #print url[4], qs
    if qs.has_key(parameter):
        link = qs[parameter][0]
        url = urlparse.urlparse(link, 'http')
        #print '-->', link
        return (link, url)
    return (None, None)

def parse_search_results(html):
    if html.find('</b> - did not match any documents.  </p>') != -1: return []
    
    #pagelen = html.find('<span id="fblmi"')
    pagelen = html.find('<div id="bottomads">') #len(html)
    if pagelen == -1:
        # this is for ie6
        pagelen = html.find('</ol></div></div>')
        if pagelen == -1:
            # this is also for ie6
            pagelen = html.find('</div></div><div><div style=')
    
    start = html.find('<div id="topstuff"')
    if start == -1:
        start = html.find('<h2 class="hd">Search Results')
        if start == -1:
            # this is for ie6
            start = html.find('<div id="res">')

    positions = []
    find_all(html, start, pagelen, '<li class="g"><h3>', 'NORMAL', positions) # this only shows up for ie6
    find_all(html, start, pagelen, '<li class="g"><h3 class="r">', 'NORMAL', positions) # this only shows up for ie6
    find_all(html, start, pagelen, '<li class="g"><table class="obcontainer"', 'INFOBOX', positions) #ie6
    find_all(html, start, pagelen, '</div><h3 class="r">', 'NORMAL', positions) #ie6
    find_all(html, start, pagelen, '<li class="g"><div><h3 class="r">', 'NEWS', positions) # ie6
    find_all(html, start, pagelen, '<li class="g"><table class="ts">', 'NORMAL', positions) # ie6  -- was VIDEO, changed to better reflect non-ie6 results
    find_all(html, start, pagelen, '<li class="g" style="margin-bottom:17px">', 'NORMAL', positions) #ie6 -- was VIDEO, changed to better reflect non-ie6 results
    find_all(html, start, pagelen, '<li><h3 class="r">', 'VIDEO', positions) #ie6

    find_all(html, start, pagelen, 'class=g><div class=vsc', 'NORMAL', positions)
    find_all(html, start, pagelen, 'class="g"><div class="vsc"', 'NORMAL', positions)

    find_all(html, start, pagelen, '<div id="sx">', 'RELATED', positions)

    find_all(html, start, pagelen, '<li class="g"><h3 class="r g0">', 'DICTIONARY', positions)

    find_all(html, start, pagelen, '<li class="g" id="imagebox_bigimages"', 'IMAGE', positions)
    find_all(html, start, pagelen, '<li class=g id=imagebox_bigimages', 'IMAGE', positions)

    find_all(html, start, pagelen, '<li class="head">', 'VIDEO', positions)
    find_all(html, start, pagelen, '<li class=head>', 'VIDEO', positions)

    find_all(html, start, pagelen, '<li class=g ><h3 class=r>', 'BOOKS', positions)
    find_all(html, start, pagelen, '<li class="g"><h3 class=r>Book searches for ', 'BOOKS', positions)
    find_all(html, start, pagelen, '<li class="g"><h3 class="r">Book searches for ', 'BOOKS', positions)

    find_all(html, start, pagelen, '<li class="g"><h3 class=r>Actor searches for ', 'ACTORS', positions)
    find_all(html, start, pagelen, '<li class="g"><h3 class="r">Actor searches for ', 'ACTORS', positions)

    find_all(html, start, pagelen, '<li id="productbox" class="g">', 'PRODUCT', positions)
    find_all(html, start, pagelen, '<li id=productbox class=g>', 'PRODUCT', positions)
    find_all(html, start, pagelen, '<li class=g id=productbox', 'PRODUCT', positions)

    find_all(html, start, pagelen, '<li class="g" id="lclbox">', 'LOCALBOX', positions)
    find_all(html, start, pagelen, '<li class=g id=lclbox>', 'LOCALBOX', positions)

    find_all(html, start, pagelen, '<li class=g class=g id=newsbox>', 'NEWS', positions)
    find_all(html, start, pagelen, '<li class="g" class="g" id="newsbox">', 'NEWS', positions)
    find_all(html, start, pagelen, '<li class="g" id="newsbox">', 'NEWS', positions)
    find_all(html, start, pagelen, '<li class=g id=newsbox>', 'NEWS', positions)

    find_all(html, start, pagelen, '<li class="g"> <h3 class="r">', 'SIMILAR', positions)
    find_all(html, start, pagelen, '<li class="g"> <h3 class=r>', 'SIMILAR', positions)

    find_all(html, start, pagelen, '<li class="g"><h3 class="r inl">', 'MAPS', positions)
    find_all(html, start, pagelen, '<li class="g tpo">', 'INFOBOX', positions)

    find_all(html, start, pagelen, '<table class="ts std"><tbody><tr>', 'SCHOLAR', positions)

    find_all(html, start, pagelen, '<li class="g pplic"', 'G+PROFILE', positions)
    find_all(html, start, pagelen, '<li class="g" style="position:relative;line-height:1.24">', 'G+POST', positions)

    positions.append((pagelen, 'END'))
    positions.sort()

    #print start, pagelen, len(html)
    #print positions

    x = start
    
    links = []
    
    while True:
        x = html.find('href="', x) + 6
        
        if x < start or x > pagelen: break
        
        y = html.find('"', x)
        link = html[x:y]
        url = urlparse.urlparse(link, 'http')
        
        #print link
        
        if is_good_url(url): links.append((x, link))
        
        # protocol can be http or https!
        elif link.find('://www.google.com/url?') != -1: 
            link, url = parse_google_redirect(link, url, 'url')
            if url and is_good_url(url): links.append((x, link)) 
        elif link.startswith('/url?'): 
            link1, url1 = parse_google_redirect(link, url, 'url')
            if url1 and is_good_url(url1): links.append((x, link1))
            else:
                link2, url2 = parse_google_redirect(link, url, 'q')
                if url2 and is_good_url(url2): links.append((x, link2))
        elif link.find('://www.google.com/imgres?') != -1:
            link, url = parse_google_redirect(link, url, 'imgurl')
            if url and is_good_url(url): links.append((x, link))       
        elif link.startswith('/imgres?'):
            link, url = parse_google_redirect(link, url, 'imgurl')
            if url and is_good_url(url): links.append((x, link))       
    
    #print links

    results = []
    
    box = 1
    l = 0
    results.append((positions[box-1][1], []))
    lastlink = ''
    
    while box < len(positions):
        if links[l][0] < positions[box][0]:
            #print positions[box-1][1], '\t', links[l][1]
            if lastlink != links[l][1]:
                results[box-1][1].append(links[l][1])
                lastlink = links[l][1]
            l += 1
            if l == len(links): break
        else:
            #print links[l][0], positions[box][0]
            #print '----------------------------------------------'
            box += 1
            results.append((positions[box-1][1], []))
            lastlink = ''
    
    if len(results) != len(positions) - 1: 
        try:
            results.append((positions[-2][1], []))
        except:
            return None
    
    #print results

    # fix results that are in generic, NORMAL divs, but are actually specialized
    for i, result in enumerate(results):
        if result[0] == 'NORMAL' and len(result[1]) > 0:
            if result[1][0].find('maps.google.com/maps?') != -1: results[i] = ('LOCALBOX', result[1])
            elif result[1][0].find('www.youtube.com/watch?') != -1: results[i] = ('VIDEO', result[1])
            elif result[1][0].find('www.google.com/imgres?imgurl') != -1: results[i] = ('IMAGE', result[1])

    return results

def write_results(results, fname):
    f = open(fname, 'w')
    for name, links in results:
        f.write('%s %i' % (name, len(links)))
        for l in links:
            f.write(' %s' % (l))
        f.write('\n')
    f.close()

def print_results(results):
    for name, links in results:
        if len(links) == 0:
            print '%12s' % (name)
            continue
        print '%12s' % (name), links[0][0:80]
        for link in links[1:]:
            print '%12s' % (''), link[0:80]    

if __name__ == "__main__":
    f = open(sys.argv[1])
    html = f.read()
    results = parse_search_results(html)
    if results: print_results(results)
    else: print 'Unable to parse'
