/*
 * login_browse.js
 * 
 * by: ArashMolaviKakhki
 * 
 * Goal: Given a list of urls, browse them one by one!
 * 
 * Usage:
 * phantomjs login_browse.js --username=[] --password=[] --url_file=[] --output=[]
 * 
 * Example:
 * phantomjs login_browse.js --username=kakhkation --password=kheilikhari --url_file=url --output=Dir_Test
 * 
 * optional flags:
 *    --no_login:   if set, won't login, DEFAULT: Logs in
 *    --clear_cookies:clears cookies after each query, DEFAULT: No (won't clear cookies)
 *    --depth_limit:  depth of clicks within a domain before going to the next url
 *    --page_limit:   number of clicks within a domain before going to the next url
 *    --retry_limit:  number of retrys before skipping, DEFAULT: 3
 *    --sleep_page :  sleep time (ms) between clicks within a domain, DEFAUL: 5000
 *    --sleep_query:  sleep time (ms) between queries, DEFAUL: 5000
 *    --sleep_retry:  sleep time (ms) between retries, DEFAUL: 5000
 *    --status_file:  file that stores the latest url_term_index
 *    --user-agent:   sets the user-agent, DEFAUL: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.6"
 */

var DEBUG1 = 1;
var DEBUG2 = 1;
var DEBUG3 = 0;

var fs     = require('fs');
var system = require('system');
var page   = require('webpage').create();

page.settings.userAgent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) AppleWebKit/536.6 (KHTML, like Gecko) Chrome/20.0.1132.57 Safari/536.6";
page.customHeaders    = {"Accept-Language":"en"};

system.url_terms      = [];
system.url_term_index = 0;

system.page_limit     = 1;
system.depth_limit    = 1;
system.page_index     = 0;
system.depth_index    = 0;

system.login     = 1;
system.clear_cookies = 0;
system.sleep_page    = 5000;
system.sleep_query   = 5000;
system.sleep_retry   = 5000;

system.retry_limit = 3;
system.retry_count = 0;

system.args.forEach(function (arg, i) {
  parts = arg.split("=");
  if (parts[0] == '--username')
    system.username = parts[1];
  else if (parts[0] == '--password')
    system.password = parts[1];
  else if (parts[0] == '--url')
    system.url_terms[0] = parts[1];
  else if (parts[0] == '--no_login')
      system.login = 0;
  else if (parts[0] == '--clear_cookies')
    system.clear_cookies = 1;
  else if (parts[0] == '--depth_limit')
    system.depth_limit = parseInt(parts[1]);
    else if (parts[0] == '--sleep_page')
      system.sleep_page = parts[1];
    else if (parts[0] == '--sleep_query')
      system.sleep_query = parts[1];
    else if (parts[0] == '--sleep_retry')
      system.sleep_retry = parts[1];
    else if (parts[0] == '--output') {
    system.target_dir = parts[1];
    checkDirectory(system.target_dir);
  }
    else if (parts[0] == '--status_file'){
      system.status_file = parts[1];
      f = fs.open(system.status_file, "r");
    line = f.readLine();
    system.url_term_index = parseInt(line.replace("\n", ""))+1;
    f.close();
  }
    else if (parts[0] == '--url_file') {
      try {
        f = fs.open(parts[1], "r");
        while (line = f.readLine())
          system.url_terms.push(line.replace("\n", ""));
        f.close();
      } 
      catch (e){
        die_with_error("Error reading keywords from '" + parts[1] + "': " + e);
      }
  }
  
    else if (parts[0] == '--user-agent') {
      if (parts[1] == 'default')
        page.settings.userAgent = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1216.0 Safari/537.2";
      else if (parts[1] == 'xp')
        page.settings.userAgent = "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1216.0 Safari/537.2";
      else if (parts[1] == 'win7')
        page.settings.userAgent = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1216.0 Safari/537.2";
      else if (parts[1] == 'osx') 
        page.settings.userAgent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1216.0 Safari/537.2";
      else if (parts[1] == 'linux')
        page.settings.userAgent = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1216.0 Safari/537.2";
      else if (parts[1] == 'android')
        page.settings.userAgent = "Mozilla/5.0 (Linux; Android 4.1.1; Nexus S Build/JRO03E) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Mobile Safari/535.19";
      else if (parts[1] == 'ios')
        page.settings.userAgent = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 5_1_1 like Mac OS X; en-us) AppleWebKit/534.46.0 (KHTML, like Gecko) CriOS/19.0.1084.60 Mobile/9B206 Safari/7534.48.3";
      else if (parts[1] == 'ie6')
        page.settings.userAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727)";
      else if (parts[1] == 'ie8')
        page.settings.userAgent = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)";
      else if (parts[1] == 'chrome')
        page.settings.userAgent = "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.2 (KHTML, like Gecko) Chrome/22.0.1216.0 Safari/537.2";
      else if (parts[1] == 'firefox')
        page.settings.userAgent = "Mozilla/5.0 (Windows NT 6.1; rv:15.0) Gecko/20100101 Firefox/15.0.1";
      else if (parts[1] == 'safari')
        page.settings.userAgent = "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/537.13+ (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2"; 
    }
});

/**
 * safe way to create a directory
 * @param dir - path and name of the directory to create
 */
function checkDirectory(dir) {
  try {
    if (! fs.exists(dir))
      fs.makeDirectory(system.target_dir);
  } 
  catch (e) {
    die_with_error("Could not create directory '" + dir + "': " + e);
  }
  if (! fs.isDirectory(system.target_dir))
    die_with_error("Could not create directory '" + dir + "': Exists as a file.");
}

/**
 * Logs a message to console together with date and time
 * @param msg
 */
function log(msg) {
  console.log((new Date().getTime()) + ": " + msg);
}

/**
 * drops the content of the webpage to .html file and renders the graphical representation to .png file
 * @param content
 */
function log_html(content) {
  if (DEBUG1) log('Doing log_html ...');
  if (DEBUG1) log(system.url_terms[system.url_term_index].replace("/", "+") + "." + system.page_index + "." + system.depth_index);
  page.render(system.target_dir + "/" + system.url_terms[system.url_term_index].replace("/", "+") + "." + system.page_index + "." + system.depth_index + ".png");
  f = fs.open(system.target_dir + "/" + system.url_terms[system.url_term_index].replace("/", "+") + "." + system.page_index + "." + system.depth_index + ".html", "w");
  f.writeLine(content);
  f.close();
}

/**
 * checks if the page loaded successfully. If so, returns 1, otherwise retries or dies
 * @param status
 * @returns {Number}
 */
function check_page(status) {
  if (DEBUG1) log('Doing check_page ...' + status);
  if (! page.injectJs("jquery-1.7.2.js")) die_with_error("Error injecting Jquery");
//  page.onResourceReceived = function(resource) {
//      console.log('Loaded: ' + resource.url + ' with status ' + resource.status);
//  }
  if (status !== 'success'){
    if (system.retry_count == system.retry_limit){
      system.url_term_index++;
      system.retry_count = 0;
      f = fs.open(system.status_file, "w");
        f.writeLine(system.url_term_index.toString());
        f.close();
      log("Too many failures! Skipping the page!");
    }
    else{
      log("Sleeping and retrying...");
      system.retry_count++;
    }
    var interval = setInterval(function() {
        clearInterval(interval);
        system.depth_index = 0;
        system.page_index  = 0;
        var current_url = system.url_terms[system.url_term_index];
        next_browse(("http://"+current_url), get_results);
    }, system.sleep_retry);
    return 0;
  }
  return 1;
}

/**
 * handles the load finished event
 * @param status
 */
function load_finished(status) {
  if (DEBUG1) log('Doing load_finished ...' + status);
  var prev_load_finished = page.load_finished;
  page.load_finished = null;
  if (prev_load_finished !== null)
    prev_load_finished(status);
}

/**
 * logs a message and dies
 * @param message
 */
function die_with_error(message) {
  log(message);
  phantom.exit();
}

/**
 * Used to click on a link within a page after timeout. Callback will be called after successful load of the next page
 * @param func
 * @param callback
 */
function next_page(func, callback) {
  if (DEBUG1) log('Doing next_page ...' + status);
  if (DEBUG2) console.log('\t***sleep_page NEXT_PAGE***');
  
  var interval = setInterval(function() {
    if (DEBUG2) console.log('\t***Wake up***');
    if (DEBUG2) console.log('\tload_finished:\t' + (callback.toString()).split(" ")[1]);
    if (DEBUG2) console.log('\tfunc:\t\t' + func);
    page.load_finished = callback;
    clearInterval(interval);
    eval("function step() { " + func + " return 1; }");
    if (! page.evaluate(step))
      die_with_error("Unable to execute function '" + func + "'");
  }, system.sleep_page);
}

/**
 * gets all links which lead to another pages within the same domain
 * @param CurrentDomain
 * @returns {Array}
 */
function get_all_links(CurrentDomain){
  if (DEBUG1) log('get_all_links ... CurrentDomain: ' + CurrentDomain);
    var links = page.evaluate(function(Domain) { return jQuery.map(jQuery('a[href*=\"'+Domain+'\"]'), function(a) { return a.href; }); }, CurrentDomain);
  var ValLinks = [];
  for (var i = 0; i < links.length; i++)
    if (ValidateURL(links[i])) ValLinks.push(links[i]); 
  if (DEBUG1) console.log('\t\t#Links:' + links.length);
  if (DEBUG3) for (var i = 0; i < links.length; i++) console.log(links[i]);
  return ValLinks;
}

/**
 * checks if the given link is a redirecting link, or a javascript link
 * @param textval
 * @returns 1 if it is a redirect link, 0 if javascript
 */
function ValidateURL(textval) {
  var urlregex = new RegExp("^(http|https|ftp)\://([a-zA-Z0-9\.\-]+(\:[a-zA-Z0-9\.&amp;%\$\-]+)*@)*((25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])|([a-zA-Z0-9\-]+\.)*[a-zA-Z0-9\-]+\.(com|edu|gov|int|mil|net|org|biz|arpa|info|name|pro|aero|coop|museum|[a-zA-Z]{2}))(\:[0-9]+)*(/($|[a-zA-Z0-9\.\,\?\'\\\+&amp;%\$#\=~_\-]+))*$");
    return urlregex.test(textval);
}

/**
 * Used to browse to the next page on the list
 * @param url
 * @param callback
 */
function next_browse(url, callback) {
  if (DEBUG1) log('Doing next_browse ...' + status);
  if (DEBUG2) console.log('\t***sleep_query NEXT BROWSE***');
  var interval = setInterval(function() {
    if (DEBUG2) console.log('\t***Wake up***');
    if (DEBUG2) console.log('\tload_finished :\t' + (callback.toString()).split(" ")[1]);
    if (DEBUG2) console.log('\tpage.open(url):\t' + url);
    if (system.clear_cookies){
      phantom.clearCookies();
      if (DEBUG3) console.log('\tCookies Size: ' + phantom.cookies.length);
    }
    page.load_finished = callback;
    clearInterval(interval);
    page.open(url);
  }, system.sleep_query);
}

/**
 * logs the current page, then either clicks another link within domain or goes to the next page in the list
 * @param status
 */
function get_results(status) {
  if (DEBUG1) log('Doing get_results ...' + status);
  if (!check_page(status)) return;
  
  log_html(page.content);
  
  var AllLinks = get_all_links(system.url_terms[system.url_term_index]);
  console.log('\tAllLinks.length: ' + AllLinks.length);
  
  if ((system.depth_index < system.depth_limit) && (AllLinks.length != 0)){
    system.depth_index++;
    var randomLink = AllLinks[Math.floor(Math.random()*AllLinks.length)];
    if (randomLink == null)
      die_with_error('Could not extract links!');
    next_page("window.location.href = \"" + randomLink + "\";", get_results);
  }
  else{
    system.depth_index = 0;
    if (system.page_index < system.page_limit){
      system.page_index++;
    }
    else {
      system.page_index = 0;
      system.url_term_index++;
      f = fs.open(system.status_file, "w");
        f.writeLine(system.url_term_index.toString());
        f.close();
    }
    if (system.url_term_index < system.url_terms.length) {
      var current_url = system.url_terms[system.url_term_index];
      next_browse(("http://"+current_url), get_results);
    }
    else {
      log("Done with all search terms -- exiting");
      phantom.exit();
    }
  }
}

/**
 * login function for the google account
 * @param status
 */
function do_login(status) {
  if (DEBUG1) log('Doing do_login ...' + status);
  if (! page.injectJs("jquery-1.7.2.js")) die_with_error("Error injecting Jquery");
  if (status !== 'success') die_with_error("Couldn't login!");
  if (page.evaluate(function() { return (document.getElementById('Email') !== null); })) {
    page.evaluate(function(user, pass) { $('#gaia_loginform input#Email').val(user); $('#gaia_loginform input#Passwd').val(pass); }, system.username, system.password);
    var current_url = system.url_terms[system.url_term_index];
    next_page("$('#gaia_loginform input#signIn').click();", function(){ next_browse(("http://"+current_url), get_results); });
  }
  else
    die_with_error("Could not find login form");
}

/**
 * prints global settings
 */
function log_settings(){
  console.log('\n\tsystem.login: ' + system.login);
  console.log('\tsystem.clear_cookies: ' + system.clear_cookies + '\tCookies Size: ' + phantom.cookies.length);
  console.log('\tsystem.page_limit: ' + system.page_limit);
  console.log('\tsystem.depth_limit: ' + system.depth_limit + '\n');
}


/*----------------------------------------------------------------------------------------------
 * Main body of the code:
 ----------------------------------------------------------------------------------------------*/
log_settings();

if (system.login) {
    page.load_finished  = do_login;
    page.onLoadFinished = load_finished;
    page.open("https://accounts.google.com/ServiceLogin?hl=en&continue=https://www.google.com/");
}
else {
  var current_url = system.url_terms[system.url_term_index];
    page.load_finished  = next_browse(("http://"+current_url), get_results);;
    page.onLoadFinished = load_finished;
    page.open("http://www.google.com/");
}
/*----------------------------------------------------------------------------------------------
 ----------------------------------------------------------------------------------------------*/