# set of tools for parsing and creating figures. Do not use this file directly, use compare_pages and print_parse instead!

import os, time, math
import jellyfish
import matplotlib
import google
matplotlib.use('PDF')
import matplotlib.pyplot as plt
from matplotlib.ticker import AutoMinorLocator, MultipleLocator

personalized = ['logged_out', 'location', 'click_history']

experiments = {'OS': {'xp': 'Windows XP', 'win7': 'Windows 7 [Control]', 'osx': 'OSX', 'linux':  'Linux'},
               'gender': {'male': 'Male', 'female': 'Female [Control]', 'other': 'Other'},
               'browser':{'chrome': 'Chrome 22 [Control]', 'firefox': 'Firefox 15', 'ie6': 'IE 6',
                          'ie8':  'IE 8', 'safari': 'Safari'},
               'age':{'15': '15 [Control]', '25': '25', '35': '35', '45':  '45', '55': '55', '65': '65'},
               'logged_out':{'out': 'No Cookies [Control]', 'in': 'Logged In', 'noclear': 'Logged Out'},
               "location":{'ca':'CA','il':'IL','ma':'MA [Control]','nc':'NC','or':'OR','pa':'PA','ut':'UT',
                           'wa':'WA'},
               "browse_history":{'nothing':'No History','0-50k':'$0-50K','150k':'>$150K','18_24':'18-24',
                                 '35_44':'35-44','50-100k':'$50-100K','65':'>=65','asian':'Asian',
                                 'college':'College','female':'Female','hispanic':'Hispanic',
                                 'no_college':'No College','100-150k':'$100-150K','18':'<18','25_34':'25-34',
                                 '45_54':'45-55','55_64':'55-64','african_american':'African American',
                                 'caucasian':'Caucasian','grad_school':'Grad School','male':'Male'},
               "search_history":{'nothing':'No History','0-50k':'$0-50K','150k':'>$150K','18_24':'18-24',
                                 '35_44':'35-44','50-100k':'$50-100K','65':'>=65','asian':'Asian',
                                 'college':'College','female':'Female','hispanic':'Hispanic',
                                 'no_college':'No College','100-150k':'$100-150K','18':'<18','25_34':'25-34',
                                 '45_54':'45-55','55_64':'55-64','african_american':'African American',
                                 'caucasian':'Caucasian','grad_school':'Grad School','male':'Male'},
               "search_history2": {'bigspender':'Big Spender','christian':'Christian',
                                   'conservative':'Conservative','gay':'Gay','jewish':'Jewish',
                                   'liberal':'Liberal','muslim':'Muslim','nothing':'Nothing','tech':'Techie'},


               "click_history_rerun2":{'nothing':'No History','0-50k':'$0-50K','150k':'>$150K','18_24':'18-24',
                                       '35_44':'35-44','50-100k':'$50-100K','65':'>=65','asian':'Asian',
                                       'college':'College','female':'Female','hispanic':'Hispanic',
                                       'no_college':'No College','100-150k':'$100-150K','18':'<18','25_34':'25-34',
                                       '45_54':'45-55','55_64':'55-64','african_american':'African American',
                                       'caucasian':'Caucasian','grad_school':'Grad School','male':'Male'},
               "click_history_profiles_rerun": {'big_spender':'Big Spender','christian':'Christian',
                                                'us_conservative':'Conservative','lgbt':'LGBT','jewish':'Jewish',
                                                'us_liberal':'Liberal','muslim':'Muslim','nothing':'Nothing',
                                                'tech':'Techie'},
}

plot_experiments = {
    'OS': {'xp': 'Windows XP', 'win7': 'Windows 7 [Control]', 'osx': 'OSX', 'linux':  'Linux'},
    'gender': {'male': 'Male', 'female': 'Female [Control]', 'other': 'Other'},
    'browser': {'chrome': 'Chrome 22 [Control]', 'firefox': 'Firefox 15', 'ie6': 'IE 6', 'ie8':  'IE 8',
                'safari': 'Safari'},
    'age': {'15': '15 [Control]', '25': '25', '35': '35', '45':  '45', '55': '55', '65': '65'},
    'logged_out': {'out': 'No Cookies [Control]', 'in': 'Logged In', 'noclear': 'Logged Out'},
    'location':{'il':'IL', 'ca':'CA', 'wa':'WA','ma':'MA [Control]','nc':'NC','or':'OR','pa':'PA','ut':'UT'},
    'canary': {'control': 'Fresh vs. Used'},

    'browse_history#0' : {'nothing':'No History [Control]'},
    'browse_history#1' : {'0-50k':'$0-50K','150k':'>$150K','50-100k':'$50-100K',
                          '100-150k':'$100-150K'},
    'browse_history#2' : {'18_24':'18-24','35_44':'35-44',
                          '65':'>=65','18':'<18','25_34':'25-34','45_54':'45-55','55_64':'55-64'},
    'browse_history#3' : {'asian':'Asian','hispanic':'Hispanic',
                          'african_american':'African American','caucasian':'Caucasian'},
    'browse_history#4' : {'college':'College','no_college':'No College',
                          'grad_school':'Grad School'},
    'browse_history#5' : {'female':'Female','male':'Male'},

    'search_history#0' : {'nothing':'No History [Control]'},
    'search_history#1' : {'0-50k':'$0-50K','150k':'>$150K','50-100k':'$50-100K',
                          '100-150k':'$100-150K'},
    'search_history#2' : {'18_24':'18-24','35_44':'35-44',
                          '65':'>=65','18':'<18','25_34':'25-34','45_54':'45-55','55_64':'55-64'},
    'search_history#3' : {'asian':'Asian','hispanic':'Hispanic',
                          'african_american':'African American','caucasian':'Caucasian'},
    'search_history#4' : {'college':'College','no_college':'No College',
                          'grad_school':'Grad School'},
    'search_history#5' : {'female':'Female','male':'Male'},

    'search_history2#0' : {'nothing':'No History [Control]'},
    "search_history2#1" : {'bigspender':'Big Spender','gay':'Gay','tech':'Techie'},
    "search_history2#2" : {'christian':'Christian','jewish':'Jewish','muslim':'Muslim'},
    "search_history2#3" : {'conservative':'Conservative','liberal':'Liberal'},

    'click_history_rerun2#0' : {'nothing':'No History [Control]'},
    'click_history_rerun2#1' : {'0-50k':'$0-50K','150k':'>$150K','50-100k':'$50-100K',
                                '100-150k':'$100-150K'},
    'click_history_rerun2#2' : {'18_24':'18-24','35_44':'35-44',
                                '65':'>=65','18':'<18','25_34':'25-34','45_54':'45-55','55_64':'55-64'},
    'click_history_rerun2#3' : {'asian':'Asian','hispanic':'Hispanic',
                                'african_american':'African American','caucasian':'Caucasian'},
    'click_history_rerun2#4' : {'college':'College','no_college':'No College',
                                'grad_school':'Grad School'},
    'click_history_rerun2#5' : {'female':'Female','male':'Male'},

    'click_history_profiles_rerun#0' : {'nothing':'No History [Control]'},
    "click_history_profiles_rerun#1" : {'big_spender':'Big Spender','lgbt':'Gay','tech':'Techie'},
    "click_history_profiles_rerun#2" : {'christian':'Christian','jewish':'Jewish','muslim':'Muslim'},
    "click_history_profiles_rerun#3" : {'us_conservative':'Conservative','us_liberal':'Liberal'},

}



def start_plot(xlabel, ylabel):
    fig = plt.figure(figsize=(6,4))
    ax = fig.add_subplot(111)
    
    if xlabel: ax.set_xlabel(xlabel) #, labelpad=20)
    if ylabel: ax.set_ylabel(ylabel) #, labelpad=20)

    ax.tick_params(color='0.3', pad=12)
    for spine in ax.spines.itervalues():
        spine.set_color('0.3')

    ax.xaxis.set_minor_locator(AutoMinorLocator())
    ax.yaxis.set_minor_locator(AutoMinorLocator()) #MultipleLocator(10))

    ax.grid(True, which='minor', linestyle='-', linewidth=0.5, alpha=0.09)
    ax.grid(True, which='major', linestyle='-', linewidth=0.5, alpha=0.15)

    return (fig, ax)

def finish_plot(fig, ax, fname, xticlabels, yticlabels, xlim, xlog, ylim, ylog):
    if xlim: ax.set_xbound(xlim[0], xlim[1])
    if xlog: ax.set_xscale('log')

    if ylim: ax.set_ybound(ylim[0], ylim[1])
    if ylog: ax.set_yscale('log')

    xs, xe = ax.get_xbound()
    fudge = 0.06 * (xe - xs) / 2
    ax.set_xbound(xs - fudge, xe + fudge)

    if xticlabels:
        t = len(ax.get_xticklabels())
        if t < len(xticlabels):
            labels = []
            skip = int(math.ceil(float(len(xticlabels))/t))
            i = skip
            for l in xticlabels:
                if i == skip:
                    labels.append(l)
                    i = 0
                i += 1
            xticlabels = labels
            
        ax.xaxis.set_ticklabels(xticlabels)
    if yticlabels: ax.yaxis.set_ticklabels(yticlabels)

    ys, ye = ax.get_ybound()
    fudge = 0.06 * (ye - ys) / 2
    ax.set_ybound(ys - fudge, ye + fudge)

    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')

    for line in ax.xaxis.get_ticklines():
        line.set_marker(3)
    for line in ax.xaxis.get_minorticklines():
        line.set_marker('')

    for line in ax.yaxis.get_ticklines():
        line.set_marker(0)
    for line in ax.yaxis.get_minorticklines():
        line.set_marker('')

    #ax.set_aspect(0.3, adjustable='box')
    #plt.tight_layout()

    fig.savefig(fname, bbox_inches='tight')

def plot_line(fname, xpts, ypts, errbar=None, xlabel=None, xlim=None, xlog=False, xticlabels=None,
              ylabel=None, ylim=None, ylog=False, yticlabels=None):

    fig, ax = start_plot(xlabel, ylabel)
    
    if not errbar: line, = ax.plot(xpts, ypts, 'k-', linewidth=2)
    else: line = ax.errorbar(xpts, ypts, yerr=errbar, fmt='k-', linewidth=2)

    finish_plot(fig, ax, fname, xticlabels, yticlabels, xlim, xlog, ylim, ylog)
    
def plot_lines(fname, xpts, ylines, labels, errbars=None, xlabel=None, xlim=None, xlog=False, xticlabels=None,
              ylabel=None, ylim=None, ylog=False, yticlabels=None, legend_loc=1, legend_ncol=1):
    
    styles = ['k-', 'k--', 'k-.', 'k:', 'k-o', 'k-s', 'k-x' , 'k-D', 'k-*', 'k-p', 'k-h']
    s = 0
    
    fig, ax = start_plot(xlabel, ylabel)
    
    if not errbars: errbars = [None for ypts in ylines]
    
    for ypts, label, errbar in zip(ylines, labels, errbars):
        if not errbar: line, = ax.plot(xpts, ypts, styles[s], linewidth=2, label=label) #, marker='o')
        else: line = ax.errorbar(xpts, ypts, yerr=errbar, fmt=styles[s], linewidth=2, label=label) #, marker='o')
        s += 1
        if s == len(styles): s = 0

    ax.legend(loc=legend_loc, ncol=legend_ncol)

    finish_plot(fig, ax, fname, xticlabels, yticlabels, xlim, xlog, ylim, ylog)

def plot_linesxy(fname, xlines, ylines, labels, errbars=None, xlabel=None, xlim=None, xlog=False, xticlabels=None,
              ylabel=None, ylim=None, ylog=False, yticlabels=None, legend_loc=1, legend_ncol=1):
    
    styles = ['k-', 'k--', 'k-.', 'k:', 'k-o', 'k-s', 'k-x' , 'k-D', 'k-*', 'k-p', 'k-h']
    s = 0
    
    fig, ax = start_plot(xlabel, ylabel)
    
    if not errbars: errbars = [None for ypts in ylines]
    
    for xpts, ypts, label, errbar in zip(xlines, ylines, labels, errbars):
        if not errbar: line, = ax.plot(xpts, ypts, styles[s], linewidth=2, label=label) #, marker='o')
        else: line = ax.errorbar(xpts, ypts, yerr=errbar, fmt=styles[s], linewidth=2, label=label) #, marker='o')
        s += 1
        if s == len(styles): s = 0

    ax.legend(loc=legend_loc, ncol=legend_ncol)

    finish_plot(fig, ax, fname, xticlabels, yticlabels, xlim, xlog, ylim, ylog)
    
def plot_cdf(fname, ll, xlabel=None, xlim=None, xlog=False, xticlabels=None, ylabel='CDF'):
    total = len(ll)
    xpts = []
    ypts = []
    for i, l in enumerate(ll):
        xpts.append(l)
        ypts.append(float(i)/total*100)

    plot_line(fname, xpts, ypts, xlabel=xlabel, ylabel=ylabel, xlim=xlim, xlog=xlog, xticlabels=xticlabels, ylim=[0, 100]) 

def date_from_dir(format, directory):
    if directory[-1] == '/': directory = directory[:-1]
    x = directory.rfind('/')
    if x != -1:
        directory = directory[x + 1:]
    return time.strftime(format, time.strptime(directory, 'results_%Y_%m_%d_%H'))

def compare_pages(page1, page2, index):
    s1 = page_to_set(page1, index)
    s2 = page_to_set(page2, index)
        
    alph = pages_to_alphabet([page1, page2], index)
    str1 = page_to_string(page1, index, alph)
    str2 = page_to_string(page2, index, alph)

    if len(s1) > len(s2):
        j = float(len(s1.intersection(s2)))/len(s1)
    else:
        j = float(len(s2.intersection(s1)))/len(s2)

    return (j, jellyfish.damerau_levenshtein_distance(str1, str2))

def load_keys(dir):
    results = {}
    keys = set()

    for fname in os.listdir(dir):
        short = fname[:fname.rfind('_')]
        if short in keys: print 'Duplicate Result -->', short, results[short], fname
        keys.add(short)
        results[short] = dir + '/' + fname 

    if len(keys) != 2000:
        print 'Too few keys:', dir, len(keys)
        sys.exit()

    return (results, keys)

def load_page(fname):
    f = open(fname)
    page = []
    for line in f:
        page.append(line.split())
    f.close()
    return page

def get_link_from_box(box, index):
    if len(box) == 2 and box[0] == "NORMAL" and len(box[1]) >= index: return box[1][index]
    return box[0]

def page_to_set(page, index):
    s = set()
    for box in page:
        s.add(get_link_from_box(box, index))
    return s

def pages_to_alphabet(pages, index):
    s = set()
    for page in pages:
        for box in page:
            s.add(get_link_from_box(box, index))
    alph = {}
    for i, url in enumerate(s):
        alph[url] = chr(i + 96)
    return alph

def page_to_string(page, index, alph):
    s = ''
    for box in page:
        s += alph[get_link_from_box(box, index)]
    return s

def load_keywords(fname):
    f = open(fname)
    s = []
    for line in f:
        #line = line.strip()
        line = line[:-1] # get rid of \n
        line = line.replace('&amp;', 'and')
        line = line.replace('&', 'and') 
        line = line.replace('/', ' ')       
        s.append(line)
    f.close()
    return s

def load_keywords2(fname):
    keywords = load_keywords('keywords.txt')
    k = []
    for key in keywords:
        k.append(key.replace(' ', '+', 1))
    return k